from pathlib import Path

import typer

from .. import defaults as _defaults
from .. import env as _env

cli = typer.Typer()


@cli.command()
def check(
    sqlalchemy_url: str = typer.Option(_defaults.HITFACTORPY_SQLALCHEMY_URL, envvar=_env.HITFACTORPY_SQLALCHEMY_URL),
    alembic_dir: Path = typer.Option(_defaults.HITFACTORPY_ALEMBIC_DIR, envvar=_env.HITFACTORPY_ALEMBIC_DIR),
):
    """alembic check"""
    from alembic import command as alembic_command
    from alembic.config import Config as AlembicConfig
    from alembic.util.exc import AutogenerateDiffsDetected

    alembic_cfg = AlembicConfig()
    alembic_cfg.set_main_option("script_location", str(alembic_dir))
    alembic_cfg.set_main_option("sqlalchemy.url", sqlalchemy_url)
    try:
        alembic_command.check(alembic_cfg)
    except AutogenerateDiffsDetected as e:
        typer.echo(e)
        raise typer.Exit(1)
    typer.echo("OK: DB and code are in sync")


@cli.command()
def upgrade(
    revision: str = "head",
    sqlalchemy_url: str = typer.Option(_defaults.HITFACTORPY_SQLALCHEMY_URL, envvar=_env.HITFACTORPY_SQLALCHEMY_URL),
    alembic_dir: Path = typer.Option(_defaults.HITFACTORPY_ALEMBIC_DIR, envvar=_env.HITFACTORPY_ALEMBIC_DIR),
):
    """alembic upgrade <revision>"""
    from alembic import command as alembic_command
    from alembic.config import Config as AlembicConfig
    from alembic.util.exc import AutogenerateDiffsDetected

    alembic_cfg = AlembicConfig()
    alembic_cfg.set_main_option("script_location", str(alembic_dir))
    alembic_cfg.set_main_option("sqlalchemy.url", sqlalchemy_url)
    try:
        alembic_command.upgrade(alembic_cfg, revision)
    except AutogenerateDiffsDetected as e:
        typer.echo(e)
        raise typer.Exit(1)
    typer.echo("OK: migration upgrade applied")


@cli.command()
def downgrade(
    revision: str = "-1",
    sqlalchemy_url: str = typer.Option(_defaults.HITFACTORPY_SQLALCHEMY_URL, envvar=_env.HITFACTORPY_SQLALCHEMY_URL),
    alembic_dir: Path = typer.Option(_defaults.HITFACTORPY_ALEMBIC_DIR, envvar=_env.HITFACTORPY_ALEMBIC_DIR),
):
    """alembic downgrade <revision>"""
    from alembic import command as alembic_command
    from alembic.config import Config as AlembicConfig
    from alembic.util.exc import AutogenerateDiffsDetected

    alembic_cfg = AlembicConfig()
    alembic_cfg.set_main_option("script_location", str(alembic_dir))
    alembic_cfg.set_main_option("sqlalchemy.url", sqlalchemy_url)

    typer.confirm("Are you sure you want to downgrade?", abort=True)

    try:
        alembic_command.downgrade(alembic_cfg, revision)
    except AutogenerateDiffsDetected as e:
        typer.echo(e)
        raise typer.Exit(1)

    typer.echo("OK: migration downgrade applied")


if __name__ == "__main__":
    cli()
